<?php
namespace Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;    


/**
 * Class prepares parameters to generate BIRT reports by returning them to a URL
 * 
 * @author Grazielle Ferreira Antunes
 */
class RelDRF extends \Zeedhi\Framework\Controller\Report{
    
    protected $entityManager;
    protected $birtStrategy;
    protected $environment;
    
    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param object $reportService - Class of zeedhi required for the operation of BIRT reports.
     * @param object $entityManager - The data persists in the database
     * @param object $birtStrategy - Class of zeedhi required for the operation of BIRT reports.
     * @param object $environment - Allows access to session data
     * 
     * @return void
     * 
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */
    public function __construct ($reportService, $entityManager, $birtStrategy, $environment) {
        $this->reportService       = $reportService;
        $this->entityManager        = $entityManager;
        $this->birtStrategy        = $birtStrategy;
        $this->environment         = $environment;
    }
    
    /**
     * Maps available reports and their respective parameters that 
     * will be received by BIRT to generate the report
     * 
     * @author Grazielle Ferreira Antunes
     */
    protected $reportMapping = array (
        'GER80800' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        
                        "CDFILIAL"  => "P_CDFILIAL",
                        "DATAINI"   => "P_DATAINI",
                        "DATAFIM"   => "P_DATAFIN",
                        "NRORG"     => "P_NRORG"
                    )
            ),
            'GER80801' =>
                array (
                    'strategy' => 'BIRT',
                    'parameterMapping' =>
                        array (
                            
                            "CDFILIAL"  => "P_CDFILIAL",
                            "DATAINI"   => "P_DATAINI",
                            "DATAFIM"   => "P_DATAFIN",
                            "NRORG"     => "P_NRORG"
                        )
                )
    );
    
    /**
     * Prepare the parameters for mapping the report including and/or 
     * changing the required parameters in the request row
     * and set the report format in birtStrategy.
     * 
     * @param Request $request – Object that receives data from the caller.
     * @param Response $response –  Object that sends back data to the caller.
     * 
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row['NRORG'] = $this->environment->getNrorg();
        $row['DATAINI'] = $row['PERIODO']['formattedStart'];
        $row['DATAFIM'] = $row['PERIODO']['formattedEnd'];
        $row['__report_name'] = $row['REPORT_NAME'];
            
        $this->birtStrategy->setReportFormat($row['FORMAT']);
    }
    
    public function getDatasApuradas (DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDFILIAL', $row)){
            
            $query = "SELECT DTAPURADA PERIODO
                                FROM (
                                    SELECT DISTINCT 
                                        TO_CHAR(DTFILIALROP, 'DD/MM/YYYY') AS DTAPURADA  , DTFILIALROP
                                            FROM FILIALROP 
                                            WHERE CDFILIAL = :CDFILIAL
                                )
                            ORDER BY DTFILIALROP";
            $params = array(
                ":CDFILIAL"  => $row['CDFILIAL']
            );
            $stmt = $this->entityManager->getConnection()->prepare($query);
            $stmt->execute($params);
            $data =  $stmt->fetchAll();
            $data = $this->array_change_key_case_recursive($data);
            $response->addDataSet(new DataSet("datas_apuradas", $data));

        }
    }

    function array_change_key_case_recursive($arr) {
        return array_map(function($item){
            if(is_array($item))
                $item = $this->array_change_key_case_recursive($item);
            return $item;
        },array_change_key_case($arr));
    }
    
}  